/** @file bx.h
 *  the unique header file needed for programming with the bx-lib
 */

/**
 *
 * @mainpage bx-lib: the Basic X11 Library tuned for C beginners
 * @author Regis Barbanchon
 *
 * @image html bx-tuning-mini.jpg
 *
 * @section Goals
 * In the times I learned programming (with Omikron Basic on Atari 520 STe 
 * game machine), you could quickly enjoy programming because things were
 * kept simple. In particular, Graphism and Drawing Primitives were part
 * of the Basic language and were no more complicated than other part
 * of Basic.
 *
 * Nowadays, things are more complicated because of Windowing systems,
 * and Event driven programming. Moreover, C language is designed to
 * be kept small, so every system specific aspects of programming are
 * moved into libraries. These ones are most often designed for
 * advanced programmers and require a wide knowledge about C
 * (such as pointers, structures, pointers to functions,
 * dynamic memory allocation, bitwise logic) 
 * and about programming in general (event driven programming, 
 * client-server programming) and sometimes object-oriented designs
 * simulated with procedural programming. Moreover, these libraries
 * are generally not written with the aim to be confined to toy programing,
 * they want to be useful tools for real applications, therefore they are huge 
 * (i.e., contains lots of functions, types, and concepts)
 * so reading the documentation and getting the whole picture may takes
 * a long time even for advanced programmers.
 *
 * The bx-lib aims at removing all technicalities of graphic programming
 * in C under X11 in order to provide a quick and easy access to drawing
 * for students that are beginners in C. It is designed to be kept
 * small and simple, not necessarily optimised or complete.
 */

#ifndef BX_H___
#define BX_H___

#include "myx.h"

/*-----------------------------------------------------------------------------
 * types
 *-----------------------------------------------------------------------------
 */

/**
 * @brief the type for images and sprites.
 */
typedef struct bx_image { MyX_Sprite * stuff;      } bx_image;

/**
 * @brief the type for colors supported by the system. 
 */
typedef struct bx_color { unsigned long pixel;     } bx_color; 

/**
 * @brief the type for snapshots of instantaneous state of the whole keyboard. 
 */
typedef struct bx_key   { unsigned char bits [32]; } bx_key;

typedef struct bx_window_stuff bx_window_stuff; 
/**
 * @brief the type for top-level windows.
 */
typedef struct bx_window { bx_window_stuff * stuff; } bx_window;


typedef struct bx_dir_stuff bx_dir_stuff;
/**
 * @brief the type for opened file directories.
 */
typedef struct bx_dir { bx_dir_stuff * stuff; } bx_dir;

/**
 * @brief the type for  for snapshots of instantaneous state of the mouse.
 */

typedef struct bx_mouse {
 
  Window root, child; /* fenetre-ecran et fenetre-cible */ 
  int xAbs, yAbs;     /* coordonnees absolues (par rapport  l'ecran) */
  int xRel, yRel;     /* coordonnees relatives (par rapport  la fenetre) */
  int x   , y;        /* coordonnees utilisateur (par rapport au dessin) */
  unsigned keyMask;   /* l'etat des boutons et de alt, altgr, ctrl, shift */
} bx_mouse;


/*-----------------------------------------------------------------------------
 * bx lib main functions
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup main Library initialisation and event loop
 *
 * @{
 */

/**
 * initialize the BX-lib (at the exclusion of optionnal audio support).
 * This is the first function to call, usually in main ().
 * No other BX function can be called before this initialisation. 
 */

void bx_init (void);

/**
 * enters the infinite loop handling window events.
 * Note that you must not necessarily call this function.
 * You can for example program an animation with an active drawing loop
 * and call bx_canvas_show() each time the drawing of an animation step 
 * is complete, since bx_canvas_show() also handle pending window events.
 */

void bx_loop (void);

/**
 * pauses for an amount of time. Window events are handled during this time.
 *
 * @param millisecs the time to wait in milliseconds
 * @return nothing.
 */

INLINE void bx_wait (unsigned long millisecs);

/**
 * clean the resources used by BX-lib when you finished using it.
 */

void bx_clean (void);

/**
 * @}
 */

/*-----------------------------------------------------------------------------
 * top-level window
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup windows Windows
 *
 * @{
 */

/**
 * creates and displays a top-level window with a specified location
 * on the screen, a specified name in the title bar, and a specified
 * canvas size.
 *
 * @param name the name in the title bar.
 * @param x,y the location of the top left corner of the title bar.
 * @param w,h the size (width and height) of the canvas of the window.
 * @return the created window.
 * @remarks
 *  - the window is displayed at creation time.
 *  - the initial size of the window is the size of the canvas. 
 *  - the canvas size is not the canvas window but the drawable area
 *    available inside the window, which is fixed once the window is created.
 *  - the canvas is always centered in the window.
 */
bx_window bx_create_window (char const name [], int x, int y, 
			    unsigned w, unsigned h);


/**
 * destroys a window.
 * 
 * @param window the window to be destroyed.
 * @return nothing.
 */

void bx_destroy_window (bx_window window);

/**
 * gets the name displayed in the title bar of window
 *
 * @ param window the window whose title is queried.
 *
 * @return the queried name. This pointer points to a string 
 * owned by the function. You cannot free it and its contents
 * changes at each call. Therefore, you should strcpy() the result
 * to your own allocated memory if yu want a persistent result.
 */

INLINE char * bx_window_name (bx_window window);

/**
 * changes the title of the window displayed in its title bar.
 *
 * @param window the window to be renamed
 * @param name the new name for the window
 * @return nothing.
 */

INLINE void bx_rename_window (bx_window window, char const name []);

/**
 * gets the width of the window.
 *
 * @param window the window whose size is queried
 * @return the width of the window in pixels
 */

INLINE unsigned bx_window_width  (bx_window window);

/**
 * gets the height of the window.
 *
 * @param window the window whose size is queried
 * @return the height of the window in pixels
 */

INLINE unsigned bx_window_height (bx_window window);

/**
 * changes the width and height of the window.
 *
 * @param window the window to be resized
 * @param w the new width in pixels for the window
 * @param h the new height in pixels for the window
 * @return nothing.
 */

INLINE void bx_resize_window (bx_window window, unsigned w, unsigned h);

/**
 * gets the x-coordinate of the top-left corner of the window in the screen.
 *
 * @param window the window whose location is queried
 * @return the x-coordinate of the window in pixels
 */

INLINE int bx_window_x (bx_window window);

/**
 * gets the y-coordinate of the top-left corner of the window in the screen.
 *
 * @param window the window whose location is queried
 * @return the y-coordinate of the window in pixels
 */

INLINE int bx_window_y (bx_window window);

/**
 * changes the location of the window in the screen.
 * Recall that pixel (0,0) is in top-left cormer, 
 * and that y-coordinates increase downwards.
 *
 * @param window the window to be moved
 * @param x the new x-coordinate for the window
 * @param y the new y-coordinate for the window
 * @return nothing.
 */

INLINE void bx_move_window   (bx_window window, int x, int y);


/**
 * @}
 */

/*-----------------------------------------------------------------------------
 * the canvas is the drawing area of a window
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup canvas Canvas (the area of a window where you can draw)
 * @{
 */


/**
 * gets the x-coordinate of the location of the top-left corner 
 * of the canvas inside its window.
 *
 * @param win the window whose canvas location is queried
 * @return x the x-coordinate of the canvas 
 */
INLINE int bx_canvas_x (bx_window win);


/**
 * gets the y-coordinate of the location of the top-left corner 
 * of the canvas inside its window.
 *
 * @param win the window whose canvas location is queried
 * @return the y-coordinate of the canvas 
 */
INLINE int bx_canvas_y (bx_window win);

/**
 * gets the width of the canvas of a window.
 *
 * @param win the window whose canvas size is queried
 * @return the width of the canvas 
 */
INLINE unsigned bx_canvas_width  (bx_window win);

/**
 * gets the height of the canvas of a window.
 *
 * @param win the window whose canvas size is queried
 * @return the height of the canvas 
 */
INLINE unsigned bx_canvas_height (bx_window win);

/**
 * erases the canvas of a window and fills it with a background color.
 *
 * @param win the window whose canvas should be erased
 * @param color the background color used to fill the canvas 
 * @return nothing.
 */
INLINE void bx_clear_canvas  (bx_window win, bx_color color);

/**
 * redraw the front buffer of the canvas of a window.
 *
 * @param win the window whose canvas should be redrawn.
 * @return nothing. 
 */
INLINE void bx_update_canvas (bx_window win);


/**
 * swaps the back buffer and front buffers of the canvas of a window,
 * therefore showing all previous drawing operations.
 *
 * @param win the window whose canvas buffers should be swapped.
 * @return nothing. 
 */
INLINE void bx_swap_canvas   (bx_window win, unsigned long millisecs);


/**
 * copy the back buffer of a canvas to its front buffer,
 * therefore showing all previous drawing operations.
 *
 * @param win the window whose back canvas buffer should be shown.
 * @return nothing. 
 */
INLINE void bx_show_canvas   (bx_window win, unsigned long millisecs);

/**
 * @}
 */

/*-----------------------------------------------------------------------------
 * color for drawing
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup bx_color Color (the main attribute of the pen)
 * @{
 * @defgroup bx_rgb Functions manipulating the RGB model
 * @{
 */

/**
 * Returns the color suported by the system that best approximates 
 * a given RGB (Red,Green,Blue) triplet. 
 * The three channels are in the range from 0 to 255
 * (or from 0x00 to 0xFF in hexadecimal base).
 *
 * @param r the red   channel, in the range from 0x00 to 0xFF (saturated).
 * @param g the green channel, in the range from 0x00 to 0xFF(saturated).
 * @param b the blue  channel, in the range from 0x00 to 0xFF (saturated).
 * @return the best matching color for the (r,g,b) triplet that is supported
 *         by the system.
 *
 * @remarks
 * - currently, only TrueColor and PseudoColor video modes are supported.
 * - in TrueColor, the bit-pattern of a  pixel value directly encodes
 *   the (R,G,B) triplet of its color. The most common encodings are
 *   24 bits per pixel (= 8+8+8 bits, that is a 256*256*256 RGB cube) 
 *   and 16 bits per pixel (= 5+6+5 bits, that is a 32*64*32 RGB cube)
 *   with the green channel having one more bit than red and blue). 
 * - in PseudoColor, pixel values are 8-bits values that are indexes
 *   in a pre-allocated 256 colors palette.
 *   The bx-lib pre-allocates a palette such that you have an 8*8*4 
 *    "poor-man" RGB cube stored in it, 
 *    making any bit-pattern of index a kind of degenerate TrueColor encoding
 *    with 8 bits per pixels (= 3+3+2 bits, the 
 *    blue channel having 1 bit less than red and green).
 * - (0xFF,0x00,0x00), (0x00,0xFF,0x00), and (0x00,0x00,0xFF) 
 *    respetively yield red, green and blue.
 * - (0xFF,0xFF,0x00), (0xFF,0x00,0xFF), and (0x00,0xFF,0xFF) 
 *   respectively yield yellow, magenta and cyan.
 * - (0x00,0x00,0x00), (0x80,0x80,0x80) and (0xFF,0xFF,0xFF)
 *   respectively yield black, gray and white.
 * @image html "bx_rgb_to_color.jpg"
 * @include    "bx_rgb_to_color.c"
 */
INLINE bx_color bx_rgb_to_color (unsigned char r, 
				 unsigned char g, 
				 unsigned char b);

int bx_red_depth (void);
int bx_green_depth (void);
int bx_blue_depth (void);

int bx_red_shift (void);
int bx_green_shift (void);
int bx_blue_shift (void); 

/**
 * Computes an approximation of the color c averaging two colors c1 and c2, 
 * that is if c1=(r1,g1,b1) and c2=(r2,g2,b2) as (Red,Green,Blue) triplets, 
 * then c=((r1+r2)/2, (g1+g2)/2,(b1+b2)/2).
 *
 * @param c1 the first color.
 * @param c2 the second color.
 * @return the mean color c of c1 and c2.
 * @image html "bx_mean_color.jpg"
 * @include    "bx_mean_color.c"
 */
INLINE bx_color bx_mean_color (bx_color c1, bx_color c2);

/**
 * Computes an approximation of the color c of an alpha-weighted 
 * blend of two colors c1 and c2, 
 * that is if c1=(r1,g1,b1) and c2=(r2,g2,b2) as (Red,Green,Blue) triplets, 
 * then c=(r1+(r2-r1)*alpha, g1+(g2-g1)*alpha, b1+(b2-b1)*alpha) 
 * for some given alpha in the range (0.0, 1.0).
 * @param c1 the first color.
 * @param c2 the second color.
 * @param alpha a blending coefficient in the range (0.0, 1.0).
 * @return the blend of c1 and c2 by coefficient alpha. 
 * @remarks
 * - for alpha=0.0, c= c1.
 * - for alpha=1.0, c= c2. 
 * - for alpha=0.5, c= bx_mean_color(c1,c2).
 * @image html "bx_blend_color.jpg"
 * @include    "bx_blend_color.c"
 */
INLINE bx_color bx_blend_color (bx_color c1, double alpha, bx_color c2);

/**
 * @}
 */ 

/*-----------------------------------------------------------------------------
 * some useful colors
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup gray_colors Predefined neutral RGB colors
 *
 * @brief Neutral RGB colors are the ones whose (r,g,b) triplets
 * verify r = g = b, that is, black, white, and all levels of
 * gray in  between.
 *
 * @{
 */

/**
 * @brief another name for bx_rgb_to_color (0xFF,0xFF,0xFF);
 */
INLINE bx_color bx_white(void);

/**
 * @brief another name for bx_rgb_to_color (0x00,0x00,0x00);
 */
INLINE bx_color bx_black(void);

/**
 * @brief another name for bx_rgb_to_color (0x80,0x80,0x80);
 * which is bx_mean_color (bx_black (), bx_white ()); 
 */
INLINE bx_color bx_gray (void);

/**
 * @brief another name for bx_rgb_to_color (0xC0,0xC0,0xC0); 
 * which is bx_mean_color (bx_gray (), bx_white ()); 
 */
INLINE bx_color bx_pale_gray(void);

/**
 * @brief another name for bx_rgb_to_color (0x40,0x40,0x40); 
 * which is bx_mean_color (bx_gray (), bx_black ()); 
 */
INLINE bx_color bx_dark_gray(void);

/**
 * @}
 * @defgroup primary_colors Predefined primary RGB colors
 *
 * @brief Primary RGB colors are the three colors whose (r,g,b) triplets
 *        have one saturated channel and twos zeroed channels,
 *        that is, red, green and blue.
 * @{
 */

/**
 * @brief another name for bx_rgb_to_color (0xFF,0x00,0x00);  
 */
INLINE bx_color bx_red (void); 

/**
 * @brief another name for bx_rgb_to_color (0x00,0xFF,0x00); 
 */
INLINE bx_color bx_green (void); 

/**
 * @brief another name for bx_rgb_to_color (0x00,0x00,0xFF);
 */
INLINE bx_color bx_blue (void); 

/**
 * @brief another name for bx_rgb_to_color (0xFF,0x80,0x80);  
 * which is bx_mean_color (bx_red (), bx_white ());   
 */
INLINE bx_color bx_pale_red (void);

/**
 * @brief another name for bx_rgb_to_color (0x80,0xFF,0x80); 
  *  which is bx_mean_color (bx_green (), bx_white ()); 
 */
INLINE bx_color bx_pale_green(void);

/**
 * @brief another name for bx_rgb_to_color (0x80,0x80,0xFF); 
 *  which is bx_mean_color (bx_blue (), bx_white ()); 
 */
INLINE bx_color bx_pale_blue (void);

/**
 * @brief another name for bx_rgb_to_color (0x80,0x00,0x00);
 * which is bx_mean_color (bx_red (), bx_black ());   
 */
INLINE bx_color bx_dark_red (void);


/**
 * @brief another name for bx_rgb_to_color (0x00,0x80,0x00);
 *  which is bx_mean_color (bx_green (), bx_black ()); 
 */
INLINE bx_color bx_dark_green(void);


/**
 * @brief another name for bx_rgb_to_color (0x00,0x00,0x80); 
 * which is bx_mean_color (bx_blue (), bx_black ());   
 */
INLINE bx_color  bx_dark_blue (void);


/**
 * @}
 * @defgroup pre_secondary_colors Predefined secondary RGB colors
 *
 * @brief Secondary RGB colors are the three RGB colors (namely       
 *        cyan, magenta and yellow) obtained by computing the sum 
 *        of two of the three primary RGB colors (namely red, green, blue).
 *        They have two saturated channel and one zeroed channel.  
 * @{
 */


/**
 * @brief another name for bx_rgb_to_color (0xFF,0xFF,0x00);
 * which is bx_add_color (bx_red (), bx_green ());   
 */
INLINE bx_color bx_yellow (void); 

/**
 * @brief another name for bx_rgb_to_color (0x00,0xFF,0xFF);
 * which is bx_add_color (bx_green (), bx_blue ());   
 */
INLINE bx_color bx_cyan (void); 

/**
 * @brief another name for bx_rgb_to_color (0xFF,0x00,0xFF);
 * which is bx_add_color (bx_red (), bx_blue ());   
 */
INLINE bx_color bx_magenta (void); 

/**
 * @brief another name for bx_rgb_to_color (0xFF,0xFF,0xC0);
 * which is bx_mean_color (bx_yellow (), bx_white ());   
 */
INLINE bx_color bx_pale_yellow (void);

/**
 * @brief another name for bx_rgb_to_color (0xC0,0xFF,0xFF); 
 * which is bx_mean_color (bx_cyan (), bx_white ()); 
 */
INLINE bx_color bx_pale_cyan (void);

/**
 * @brief another name for bx_rgb_to_color (0xFF,0xC0,0xFF);
 * which is bx_mean_color (bx_magenta (), bx_white ()); 
 */
INLINE bx_color bx_pale_magenta (void); 

/**
 * @brief another name for bx_rgb_to_color (0x80,0x80,0x00); 
 * which is bx_mean_color (bx_yellow (), bx_black ()); 
 */
INLINE bx_color bx_dark_yellow (void);

/**
 * @brief another name for bx_rgb_to_color (0x00,0x80,0x80); 
 * which is bx_mean_color (bx_cyan (), bx_black ()); 
 */
INLINE bx_color  bx_dark_cyan (void);

/**
 * @brief another name for bx_rgb_to_color (0xC0,0x00,0xC0);
 * which is bx_mean_color (bx_magenta (), bx_black ()); 
 */
INLINE bx_color bx_dark_magenta (void); 

/**
 * @}
 * @defgroup pre_tertiary_colors Predefined tertiary RGB colors
 *
 * @brief Tertiary RGB colors are the six colors (namely
 *        pink, orange, violet, azure, spring, lawn)  
 *        obtained by computing the mean between 
 *        one of the three primary RGB colors
 *        (red, green, blue) and one the three secondary RGB colors
 *        (cyan, magenta, yellow) at the exclusion of the complementary color
 *        that would result into neutral gray.
 * @{
 */


/**
 * @brief another name for bx_rgb_to_color (0xFF,0x00,0x80);
 * which is bx_mean_color (bx_red (), bx_magenta ()); 
 */
INLINE bx_color bx_pink (void); 

/**
 * @brief another name for bx_rgb_to_color (0xFF,0x80,0x00);
 * which is bx_mean_color (bx_red (), bx_yellow ()); 
 */
INLINE bx_color bx_orange (void); 

/**
 * @brief another name for bx_rgb_to_color  (0x80,0x00,0xFF);
 * which is bx_mean_color (bx_blue (), bx_magenta ()); 
 */
INLINE bx_color bx_violet (void); 

/**
 * @brief another name for bx_rgb_to_color (0x00, 0x80,0xFF);
 * which is bx_mean_color (bx_blue (), bx_cyan ());
 */
INLINE bx_color bx_azure (void); 

/**
 * @brief another name for bx_rgb_to_color (0x00, 0xFF,0x80);
 * which is bx_mean_color (bx_green (), bx_cyan ());
 */
INLINE bx_color bx_spring (void); 

/**
 * @brief another name for bx_rgb_to_color (0x80, 0xFF,0x00);
 * which is bx_mean_color (bx_green (), bx_yellow ());
 */
INLINE bx_color bx_lawn (void); 

/**
 * @brief another name for bx_rgb_to_color (0xFF,0x80,0xC0);
 * which is bx_mean_color (bx_pink (), bx_white ());
 */
INLINE bx_color bx_pale_pink (void);
 
/**
 * @brief another name for bx_rgb_to_color (0xFF,0xC0,0x80);
 * which is bx_mean_color (bx_orange (), bx_white ());
 */
INLINE bx_color bx_pale_orange (void); 

/**
 * @brief another name for bx_rgb_to_color (0xC0,0x80,0xFF);
 * which is bx_mean_color (bx_violet (), bx_white ());
 */
INLINE bx_color bx_pale_violet (void); 

/**
 * @brief another name for bx_rgb_to_color (0x80, 0xC0,0xFF); 
 * which is bx_mean_color (bx_azure (), bx_white ());
 */
INLINE bx_color bx_pale_azure (void); 

/**
 * @brief another name for bx_rgb_to_color (0x80, 0xFF,0xC0); 
 * which is bx_mean_color (bx_spring (), bx_white ());
 */
INLINE bx_color bx_pale_spring (void); 

/**
 * @brief another name for bx_rgb_to_color (0xC0, 0xFF,0x80); 
 * which is bx_mean_color (bx_lawn (), bx_white ());
 */
INLINE bx_color bx_pale_lawn (void); 

/**
 * @brief another name for bx_rgb_to_color (0x80,0x00,0x40);
 * which is bx_mean_color (bx_pink (), bx_black ());
 */
INLINE bx_color bx_dark_pink (void);

/**
 * @brief another name for bx_rgb_to_color (0x80,0x40,0x00);  
 * which is bx_mean_color (bx_orange (), bx_black ());
 */
INLINE bx_color bx_dark_orange (void);

/**
 * @brief another name for bx_rgb_to_color (0x40,0x00,0x80);
 * which is bx_mean_color (bx_violet (), bx_black ());
 */
INLINE bx_color bx_dark_violet (void);

/**
 * @brief another name for bx_rgb_to_color (0x00, 0x40,0x80);
 * which is bx_mean_color (bx_azure (), bx_black ());
 */
INLINE bx_color bx_dark_azure (void);

/**
 * @brief another name for bx_rgb_to_color (0x00, 0x80,0x40); 
 * which is bx_mean_color (bx_spring (), bx_black ());
 */
INLINE bx_color bx_dark_spring (void);

/**
 * @brief another name for bx_rgb_to_color (0x40, 0x80,0x00);
 * which is bx_mean_color (bx_lawn (), bx_black ());
 */
INLINE bx_color bx_dark_lawn (void);

/**
 * @}
 * @}
 */

 
/*-----------------------------------------------------------------------------
 * pen attributes
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup pen Pen (what you use to draw on a canvas)
 *
 * @{
 */

/**
 * @defgroup pen_enum Symbolic constants for pen attributes
 *
 * @{
 */

/**
 * Symbolic constants for the special effect fx in function bx_draw_image_fx()
 * and all drawing functions via bx_set_fx().
 * Each bit of an opaque pixel of the source (noted A) is combined
 * with the corresponding bit of the underlying pixel of the canvas (noted B) 
 * via a boolean function F(A,B). 
 *
 * The 4 entries of the truth table 
 * (F(0,0),F(0,1),F(1,0),F(1,1)) of this function F 
 * correspond to the binary representation (fx3,fx2,fx1,fx0) 
 * of the integer fx.
 *
 * For example, if fx=7, then fx is written (0,1,1,1) in binary, which means
 * F(0,0)=0 and F(A,B)=1 in any other case, that is, F(A,B)= A AND B.
 * This produces a "burn" effect: the bits of the image and the canvas 
 * combine into a bitwise AND to produce darker pixels.
 *
 * Similarily, if fx=1, then fx is written (0,0,0,1) in binary, which means
 * F(1,1)=0 and F(A,B)=0 in any other case, that is, F(A,B)= A OR B.
 * This produces a "glow" effect: the bits of the image and the canvas 
 * combine into a bitwise OR to produce lighter pixels.
 *
 * @remarks
 * - This naturally holds in TrueColor mode where the bit-pattern of a 
 *  pixel value directly encodes the (R,G,B) triplet of its color
 *  (8+8+8=24 bits  and 5+6+5=16 bits are the most common pixel depths).
 * - This also holds in PseudoColor where pixel values are 8-bits indexes 
 *   in a 256 colors palette because bx-lib stores a 8*8*4 "poor-man" RGB cube
 *   in this palette, in such a way that the bit pattern of an index value 
 *   can be interpreted as a degenerated direct TrueColor encoding 
 *   on 3+3+2 bits.
 */

typedef enum bx_effect { 
  BX_ZERO       =GXclear, 
  /**< (= 0 = 0000) black shadow */       
  BX_A_AND_B    =GXand, 
  /**< (= 1 = 0001) the "burn" effect */ 
  BX_A_AND_NOT_B=GXandReverse,  
  /**< (= 2 = 0010) */  
  BX_A          =GXcopy, 
  /**< (= 3 = 0011) the default, the source is copied no special effect */ 
  BX_NOT_A_AND_B=GXandInverted,  
  /**< (= 4 = 0100) */ 
  BX_B          =GXnoop,    
  /**< (= 5 = 0101) nothing happens, the destination is coied over itself */ 
  BX_A_XOR_B    =GXxor,  
  /**< (= 6 = 0110) */ 
  BX_A_OR_B     =GXor, 
  /**< (= 7 = 0111) the "glow" effect */ 
  BX_A_NOR_B    =GXnor, 
  /**< (= 8 = 1000) the video-inversion of the "glow" effect */ 
  BX_A_EQV_B    =GXequiv,    
  /**< (= 9 = 1001) */ 
  BX_NOT_B      =GXinvert, 
  /**< (=10 = 1010) video-inversion of the destination */ 
  BX_A_OR_NOT_B =GXorReverse, 
  /**< (=11 = 1011) */ 
  BX_NOT_A      =GXcopyInverted, 
  /**< (=12 = 1100)  video-inversion of the source */ 
  BX_NOT_A_OR_B =GXorInverted, 
  /**< (=13 = 1101) */ 
  BX_A_NAND_B   =GXnand,   
  /**< (=14 = 1110) the video-inversion of the "burn" effect */ 
  BX_ONE        =GXset
  /**< (=15 = 1111) white shadow */ 
} bx_effect; 

/** 
 * The type passed to bx_set_set_line_style()
 * to specify whether contour lines should be drawn plain solid
 * or dashed. Dashed line patterns either alternate between the principal
 * and secondary color (set with bx_set_color() and bx_set_color2()) 
 * or alternate between the principal color and no color.
 */
typedef enum bx_line_style {
  BX_SOLID_LINE      = LineSolid,       /**< plain solid line */
  BX_DASH2_LINE= LineDoubleDash,  
  /**< dashed line alternating between with principal and secondary color */
  BX_DASH_LINE= LineOnOffDash
  /**< dashed line alternating between with principal color and no color */
} bx_line_style;

/**
 * The type passed to bx_set_fill_style()
 * to specify how interiors of shapes are painted when those are filled.
 * Two styles are supported: plain solid fill with the principal color
 * (set with bx_set_color()) or "bathroom tile" style with a repeating pattern
 * described by an image (set with bx_set_tile_image()).
 */
typedef enum bx_fill_style {
  BX_SOLID_FILL= FillSolid, /**< plain solid fill with principal color */ 
  BX_TILED_FILL= FillTiled  /**< bathroom tiles with an image */ 
} bx_fill_style;

/**
 * The type passed to bx_set_fill_rule() and some interior test functions
 * such as bx_in_polygon(),
 * specifying which rule is used to determine if a point belongs 
 * to the interior of self intersecting curve or polygon.
 * 
 * The two available methods both cast an horizontal ray rightwards
 * from the point to be tested. 
 * - With the even-odd method, the point
 * belongs to the interior of the shape if the ray intersects its
 * contour an odd number of times. 
 * - With the winding number method, 
 * the point belongs to the interior of the shape if the ray intersects
 * upwards and downwards portions of the contour a different number of times. 
 */
typedef enum bx_fill_rule { 
  BX_WINDING_RULE = WindingRule, /**< The winding number rule */
  BX_EVEN_ODD_RULE= EvenOddRule  /**< The even-odd rule */
} bx_fill_rule;

/**
 * The type passed to some drawing functions such as bx_draw_box()
 * to specify whether the interior of the shape should be painted or
 * the contour should drawn.
 */
typedef enum bx_fill {
  BX_NOT_FILLED= 0, /**< draw the contour  (= 0) */
  BX_FILLED= 1      /**< fill the interior (= 1) */
} bx_fill;

/**
 * The type passed to some drawing functions such as bx_draw_polygon()
 * to specify whether a polygonal
 * or curved shape is to be drawn closed or open. 
 */
typedef enum bx_closure {
  BX_NOT_CLOSED= 0, /**< open shape (= 0) */
  BX_CLOSED= 1      /**< closed shape (= 1) */
} bx_closure;


/**
 * @}
 *
 */

/**
 * @defgroup pen_set Setting the active pen attributes
 *
 * @{
 */


/**
 * Sets the line style used to draw contours of shapes
 * (solid lines, doubly dashed lines, or dashed lines).
 *
 * @param line_style the line style to activate.
 * @return nothing.
 */
INLINE void bx_set_line_style (bx_line_style line_style);


/**
 * Sets the principal color used by the pen for all drawing functions.
 *
 * @param color the new color used by the pen.
 * @return nothing.  
 * @remarks
 * - You can use bx_rgb_to_color() to get a color from a RGB specification.
 */
INLINE void bx_set_color (bx_color color); 

/**
 * Sets the secondary color used by the pen for all drawing functions.
 * The secondary color is used for doubly dashed lines.
 * @param color the new color used by the pen.
 * @return nothing.  
 * @remarks
 * - You can use bx_rgb_to_color() to get a color from a RGB specification. 
 */
INLINE void bx_set_color2 (bx_color color); 

/**
 * Sets the principal color of the pen for all drawing functions,
 * directly from a RGB triplet.
 * If you do not reuse this color later,
 * you may want to use this function instead of  bx_set_color().
 * This is just equivalent to bx_set_color (bx_rgb_to_color (r,g,b));
 *
 * @param r,g,b the red, green, blue channel of the color.
 * @return nothing.
 * @remarks
 *  - The default color of the pen is black.
 */

INLINE void bx_set_rgb (unsigned char r, 
			unsigned char g, 
			unsigned char b);

 /**
 * Sets the secondary color of the pen for all drawing functions,
 * directly from a RGB triplet.
 * If you do not reuse this color later,
 * you may want to use this function instead of  bx_set_color().
 * This is just equivalent to bx_set_color2 (bx_rgb_to_color (r,g,b));
 *
 * @param r,g,b the red, green, blue channel of the color.
 * @return nothing.
 * @remarks
 *  - The default secondary color of the pen is white.
 */
INLINE void bx_set_rgb2 (unsigned char r, 
			 unsigned char g, 
			 unsigned char b);
/**
 * Sets the line width of the pen for drawing functions
 * like bx_draw_line (), bx_draw_circle ()...
 *
 * @param size the new width of the pen
 * @return nothing. 
 * @remarks
 *  - The default line width of the pen is 0.
 *
 * @image html "bx_set_line_width.jpg"
 * @include    "bx_set_line_width.c"
 */
INLINE void bx_set_line_width (unsigned int size); 


/**
 * Sets the fill style used to paint the interior of shapes.
 * Two styles are supported : plain solid fill style and
 * "bathroom tile" style using an image as a repeating motif.
 *
 * @param fill_style the fill style to set as active
 * @return nothing.
 * @remarks
 * - use bx_set_tile_image() to set the repeating motif and
 * - use bx_set_tile_origin() to set the origin of a tile.
 */
INLINE void  bx_set_fill_style (bx_fill_style fill_style);

/**
 * Sets the repeating "bathroom tile" pattern to use
 * for filling shapes if bx_set_fill_style() was called with BX_TILED_FILL.
 * @param the image to use as a repeating pattern
 * @return nothing.
 */

INLINE void  bx_set_tile_image  (bx_image image);

/**
 * Shifts the locations of repeating "bathroom tile" patterns used
 * for filling shapes if bx_set_fill_style() was called with BX_TILED_FILL.
 * @param x x-coordinate of the top-left-corner of the starting tile
 * @param y y-coordinate of the top-left-corner of the starting tile
 * @return nothing.
 */

INLINE void  bx_set_tile_origin (int x, int y);
 
/**
 * Sets how a self-intersecting polygon should be filled.
 * Two rules are possible: The winding rule and the odd-even rule.
 * @param fill_rule the rule to apply for next drawing functions
 * @return nothing.
 */

INLINE void bx_set_fill_rule (bx_fill_rule fill_rule);


/**
 * Sets a special effect selected among 16 for all drawing functions 
 * with the notable exception of bx_draw_image().
 *
 * @param fx the special effect to apply, in the range from 0 to 15.
 * @return nothing.
 * @remarks 
 *  - See bx_effect for details.
 *  - The default fx is BX_A, that is, the source is copied on the destination.
 *  - bx_set_effect() has no effect on bx_draw_image(). This is because
 *    bx-lib has to deal transparent and opaque images differently
 *    with special effect.
 *    Use bx_draw_image_fx() instead to get the expected result.
 */
INLINE void
bx_set_effect (bx_effect fx);

/**
 * @}
 * @}
 */

 
/*----------------------------------------------------------------------------
 * simple drawing on canvas
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup draw Drawing (what you can do with a pen on a canvas)
 * @{
 * @defgroup draw1 Drawing simple shapes (points, lines, boxes, circles...)
 * @{
 */

/**
 *
 * Draws a point on the canvas of a window.
 *
 * @param win the window owning the targeted canvas.
 * @param x,y the coordinates of the point to draw.
 * @return nothing
 *
 * @remarks
 * - the coordinates are relative to the top-left corner of the canvas.
 * - the y-axis is oriented downward.
 * - the updated canvas will be shown after the next call to bx_show_canvas().
 * - the color is set with bx_set_color() or bx_set_rgb().
 */
INLINE void bx_draw_point(bx_window win, int x, int y);

/**
 * Draws a line segment on the canvas of a window.
 *
 * @param win the window owning the targeted canvas.
 * @param x1,y1 the coordinates of the 1st end-point of the segment.
 * @param x2,y2 the coordinates of the 2nd end-point of the segment.
 * @return nothing
 *
 * @remarks
 * - the coordinates are relative to the top-left corner of the canvas.
 * - the y-axis is oriented downward.
 * - the updated canvas will be shown after the next call to bx_show_canvas().
 * - the color is set with bx_set_color() or bx_set_rgb().
 * - the line width is set with bx_set_line_width().
 *
 * @image html "bx_draw_line.jpg"
 * @include "bx_draw_line.c"
 */
INLINE void bx_draw_line(bx_window win, int x1, int y1, int x2, int y2);

/**
 * Draws a rectangle on the canvas of a window,
 * whose interior may be filled or left empty.
 *
 * @param win the window owning the targeted canvas.
 * @param x,y the coordinates of the top left corner of the rectangle.
 * @param w,h the width and height (minus 1) of the rectangle, 
 *            so that the bottom-right corner 
 *            of the bounding-box is located at (x+w, y+h).   
 * @param filled whether the interior of the rectangle
 *        is filled (no-zero) or left empty (zero)
 * @return nothing
 *
 * @remarks
 * - the coordinates are relative to the top-left corner of the canvas.
 * - the y-axis is oriented downward.
 * - the updated canvas will be shown after the next call to bx_show_canvas().
 * - the color is set with bx_set_color() or bx_set_rgb().
 * - the line width is set with bx_set_line_width().
 */
INLINE void bx_draw_box (bx_window win, int x, int y, 
			 unsigned w, unsigned h, bx_fill filled);

/**
 * Draws an ellipse on the canvas of a window,
 * whose interior may be filled or left empty.
 *
 * @param win the window owning the targeted canvas.
 * @param x,y the coordinates of the top left corner of the 
 *             bounding box of the ellipse.
 * @param w,h the width and height (minus 1) of the bounding box 
 *             of the ellipse, so that the bottom-right corner 
 *             of the bounding-box is located at (x+w, y+h).    
 * @param filled whether theellipse is filled (non zero) or empty (zero)
 * @return nothing
 *
 * @remarks
 * - the coordinates are relative to the top-left corner of the canvas.
 * - the y-axis is oriented downward. 
 * - the updated canvas will be shown after the next call to bx_show_canvas().
 * - the color is set with bx_set_color() or bx_set_rgb().
 * - the line width is set with bx_set_line_width().
 */
INLINE void bx_draw_circle (bx_window win, int x, int y, 
			    unsigned w, unsigned h, bx_fill filled);

/**
 * Draws an arc on the canvas of a window,
 * that is a portion of an ellipse between two angles, and
 * whose interior may be filled (to look as a pie slice) or left empty.
 *
 * @param win the window owning the targeted canvas.
 * @param x,y the coordinates of the top left corner of the 
 *             bounding box of the underlying ellipse of the arc.
 * @param w,h the width and height (minus 1) of the bounding box of the 
 *             underlying ellipse of the arc, 
 *	       so that the bottom-right corner of the box
 *             is located at (x+w, y+h).    
 * @param angle the starting angle of the arc, expressed in degrees
 *              from the 3 o'clock position 
 *              on the underlying ellipse of the arc. 
 * @param dangle the angle between the starting angle and the ending angle
 *               of the arc expressed in degrees
 * @param filled whether theellipse is filled (non zero) or empty (zero)
 * @return nothing
 *
 * @remarks
 * - the coordinates are relative to the top-left corner of the canvas.
 * - the y-axis is oriented downward.
 * - positive angles indicate counterclockwise motion
     while negative ones indeicate clockwise motion.
 * - if the underlying ellipse is not a circle, then the angles must be thought
     as skewed in the coordinate system of the bounding box of 
     underlying ellipse, that is, 
     an angle of 45*64 units will point toward the top right corner of
     bounding box of the underlying ellipse of the arc. 
 * - the updated canvas will be shown after the next call to bx_show_canvas().
 * - the color is set with bx_set_color() or bx_set_rgb().
 * - the line width is set with bx_set_line_width().
 */
INLINE void bx_draw_arc (bx_window win, int x, int y, 
			 unsigned w, unsigned h, 
			 double angle, double dangle, 
			 bx_fill filled);


/**
 * @}
 * @defgroup draw2 Drawing complex shapes (polygons, Bezier curves...)
 * @{
 */

/*  0   1   2   3   4   5  level
 *  4   7  13  25  49  97  bezier  length
 *  3   5   9  17  33  65  bezier2 length 
 */

#define BX_BEZIER_LEVEL  5L
#define BX_BEZIER_LENGTH    ((1L << BX_BEZIER_LEVEL)*3L + 1L)
 
#define BX_BEZIER2_LEVEL 5L
#define BX_BEZIER2_LENGTH   ((1L << BX_BEZIER_LEVEL)*2L + 1L)

#define BX_MAX_POLYGON_POINTS 1024

#define BX_MAX_CURVE_POINTS 100L
#define BX_CURVE_LENGTH     (BX_MAX_CURVE_POINTS * BX_BEZIER_LENGTH)  

#define BX_MAX_CURVE2_POINTS 100L
#define BX_CURVE2_LENGTH    (BX_MAX_CURVE2_POINTS * BX_BEZIER2_LENGTH)  

#define BX_MAX_CURVEI_POINTS 100L
#define BX_CURVEI_LENGTH    (BX_MAX_CURVEI_POINTS * BX_BEZIER_LENGTH)  

int bx_in_polygon  (int x, int y, 
		    int n, int xpoly[], int ypoly[],  
		    bx_fill_rule fill_rule);
int bx_on_polyline (int x, int y,
		    int n, int xpoly[], int ypoly[], 
		    bx_closure closed);

/**
 * Draws the contour or the interior of a polygon on the canvas of a window.
 * @param win the window owning the targetted canvas.
 * @param n the number of points in the polygon.
 * @param x the array of x-coordinates of the n points.
 * @param y the array of y-coordinates of the n points.
 * @param closed whether contour of the polygon should be closed or not. 
 * @param filled either fills the interior (=1) or draws the contour (=0).
 */
INLINE void bx_draw_polygon (bx_window win, int n, int x [], int y [], 
			     bx_closure closed, bx_fill filled);


int
bx_points_to_bezier (int x0, int y0,
		     int x1, int y1, 
                     int x2, int y2, 
		     int x3, int y3,
                     int x_res [], int y_res []);

int
bx_points_to_bezier2 (int x0, int y0, 
                      int x1, int y1, 
                      int x2, int y2,
                      int x_res [], int y_res []);

int
bx_points_to_curve (int n, int x [], int y [], bx_closure closed, 
		    int x_res[], int y_res[]);

int
bx_points_to_curve2 (int n, int x [], int y [], bx_closure closed, 
		     int x_res[], int y_res[]);


int
bx_points_to_curvei (int n, int x [], int y [], 
		     bx_closure closed, double beta,
                     int x_res[], int y_res[]);

/**
 * Draws a Bezier curve of 3rd order (controlled by 4 points)
 */
void bx_draw_bezier (bx_window window,
		     int x0, int y0, 
		     int x1, int y1, 
		     int x2, int y2, 
		     int x3, int y3);


/**
 * Draws a Bezier curve of 2nd order (controlled by 3 points)
 */
void
bx_draw_bezier2 (bx_window window,
                 int x0, int y0,
                 int x1, int y1, 
                 int x2, int y2);

/**
 * Draws a approximation curve controlled by n points
 * and made of a smooth succession Bezier curves of 3rd order.
 */
void bx_draw_curve (bx_window window, int n, int x [], int y [],
		    bx_closure closed, bx_fill filled); 

/**
 * Draws a approximation curve controlled by n points
 * and made of a smooth succession Bezier curves of 2nd order.
 */
void bx_draw_curve2 (bx_window window, int n, int x [], int y [],
		     bx_closure closed, bx_fill filled);

/**
 * Draws a interpolation curve controlled by n points
 * and made of a smooth succession Bezier curves of 3rd order.
 */
void 
bx_draw_curvei (bx_window window, int n, int x [], int y [], 
		bx_closure closed, bx_fill filled, double beta);

/**
 * @}
 * @defgroup text Drawing Text
 * @{
 */

/**
 * The "poor man" solution to draw a text on the canvas of a window.
 * Draws a text with the default font of X11.
 * The font and its size cannot be changed,
 * however the text is drawn with the pen color.
 *
 * @param win the window owning the targeted canvas.
 * @param x,y the location of the anchor of the text on the canvas.
 * @param text the string to be drawn
 * @return nothing.
 *
 * @remarks
 * - the anchor is located at the left side of the first character, 
 *   at the level of its baseline.
 */
INLINE void bx_draw_text   (bx_window win, int x, int y, char text []);

/**
 * A frontend of bx_draw_text() allowing the same format specifiers 
 * ("%d", "%s", "%f", ...) as the printf() functions family.
 *
 * @param win the window owning the targeted canvas.
 * @param x,y the location of the anchor of the text on the canvas.
 * @param format a format string in the printf() style 
 * @param ... all extra arguments needed to match the \ %-specifiers
 *            of the format 
 * @return nothing.
 */
INLINE void bx_draw_text_f (bx_window win, int x, int y, char format [], ...);
		    
/*-----------------------------------------------------------------------------
 * loading and displaying images (and mirroring)
 *-----------------------------------------------------------------------------
 */

/**
 * @}
 * @defgroup images Drawing images
 *
 * @{
 */

/**
 * Free the memory allocated by a loaded image.
 *
 * @param image the image to be freed.
 * @return nothing.
 */

INLINE void bx_free_image (bx_image image);

/**
 * Loads an image from a file (jpg, gif, png, ...)
 *
 * @param transp whether the image is fully opaque (zero) 
 *        or has some transparency information (non zero). 
 * @param name the path and filename of the image.
 * @return the image along with its eventual transparency mask.
 *
 * @remarks
 * - on error (no memory available, file not found), the program simply exits!
 * - half transparency (alpha channel) is not handled. A pixel is either
 *   fully opaque or fully transparent.
 */

INLINE bx_image bx_load_image   (int transp, char const name []);
/**
 * Loads an image from a file (jpg, gif, png, ...) just as bx_load_image()
 * but with a printf()-like format to specify the path to the file.
 *
 * @param transp whether the image is fully opaque (zero) 
 *        or has some transparency information (non zero). 
 * @param format printf()-like format string yielding the path to the image.
 * @param ... the  arguments corresponding to the \%-pecifiers of the format.
 * @return the image along with its eventual transparency mask.
 *
 * @remarks
 * - on error (no memory available, file not found), the program simply exits!
 * - half transparency (alpha channel) is not handled. A pixel is either
 *   fully opaque or fully transparent.
 */
INLINE bx_image bx_load_image_f (int transp, char const format [], ...);

/**
 * Gets the width of the loaded image in pixels.
 * @param image the image whose size is queried
 * @return its width in pixels
 */
INLINE unsigned bx_image_width  (bx_image image);

/**
 * Gets the height of the loaded image in pixels.
 * @param image the image whose size is queried
 * @return its height in pixels
 */
INLINE unsigned bx_image_height (bx_image image);

/**
 * Sets the image in a horizontal/vertical mirrored state with respect
 * to the image state at loading time.
 *
 * @param xflip boolean stating if the image should be displayed x-mirrored
 * @param yflip boolean stating if the image should be displayed y-mirrored
 */
INLINE void bx_flip_image (bx_image image, int xflip, int yflip);

/**
 * Draws an image on the canvas of a window.
 *
 * @param win the window owning the targeted canvas.
 * @param image the image to draw on the canvas
 * @param x,y the position of tha anchor of the image on the canvas,
 *            that is, its top left corner
 * @return nothing.
 *
 * @remarks
 * - images can be loaded from a file (gif, jpg, png, ...)
 *   with bx_load_image() or  bx_load_image_f().
 * - the direction of y-axis of the canvas is downward. 
 * - if the image has transparent pixels, the underlying pixels
 *   of the canvas will still be visible. 
 * - bx_set_effect() has no effect on bx_draw_image().
 *   Use bx_draw_image_fx() instead to get analog results. 
 */
INLINE void bx_draw_image (bx_window win, bx_image image, 
			   int x, int y);

/**
 * Draws an image on the canvas of a window, eventually
 * combining the backgound of the canvas and the image
 * with 1 special effect among 16.
 *
 * @param win the window owning the targeted canvas.
 * @param image the image to draw on the canvas.
 * @param x,y the position of thr anchor of the image on the canvas,
 *            that is, its top left corner.
 * @param fx the special effect to apply, in the range from 0 to 15,
 *           see bx_effect for available effects.
 * @return nothing.
 *
 * @remarks
 * - images can be loaded from a file (gif, jpg, png, ...)
 *   with bx_load_image() or  bx_load_image_f().
 * - the direction of y-axis of the canvas is downward. 
 * - bx_draw_image_fx() with fx=3= BX_A is equivalent to bx_draw_image().
 * - fx do not apply on transparent pixels of the image:  
 *   the underlying pixels of the canvas will always be visible. 
 */
INLINE void bx_draw_image_fx (bx_window win, bx_image image, 
			      int x, int y, bx_effect fx);

/**
 * @}
 * @}
 */

/*-----------------------------------------------------------------------------
 * keyboard control
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup input Input Control (how to deal with mouse and keyboard)
 * @{
 * @defgroup keyboard Actively reading the current state of the keyboard
 * @{
 */

/**
 * Takes an instantaneous snapshot of the current state of the whole keyboard,
 * that is, the state pressed/released of all its keys.
 *
 * @return the snapshot of the keyboard
 *
 * @remarks
 * - a key-press or a key-release may be a "fake" one if auto-repeat mode 
 *   is on. It is off by default
 *  (see bx_set_autorepeat() and bx_get_autorepeat()).
 */
bx_key bx_read_key (void);

/**
 * Checks if a key known by its keyname is pressed 
 * in a given keyboard snapshot.
 * @param key a snapshot of the keyboard returned by bx_read_key().
 * @param keyname the name of a given key of the keyboard.
 * @return whether the key is pressed (= 1) or released (= 0).
 *
 * @remarks
 * - see also bx_keynames() for a list of the names of all pressed keys.
 * - if keys named "Shift" and "x" are pressed, then they are both reported
 *   as such individually, but keep in mind there is no such key named as "X"
 *   for example.
 * - You use bx_keyname_pressed() instead of bx_keycode_pressed()
 *   when you want to know if the key named "a" is pressed whatever
 *   the keyboard mapping (AZERTY, QWERTY, etc...), that is, whatever
 *   the keycode is attributed for the key named "a". 
 */
int bx_keyname_pressed  (bx_key key, char keyname []);

/**
 * Checks if a key known by its keycode is pressed 
 * in a given keyboard snapshot. The keycode is a number given to key 
 * depending on its physical location on the keyboard.
 *
 * @param key a snapshot of the keyboard returned by bx_read_key().
 * @param keycode a keycode describing the physical location of a key.
 * @return whether the key is pressed (= 1) or released (= 0).
 *
 * @remarks
 * - see also bx_keynames() for a list of the names of all pressed keys.
 * - if keys named "Shift" and "x" are pressed, then they are both reported
 *   as such individually, but keep in mind there is no such key named as "X"
 *   for example.
  * - You use  bx_keycode_pressed() instead of bx_keyname_pressed()
 *   when you want to know if this key located in the region of 
 *   the top-left corner of the keyboard is pressed, whatever keyname 
 *   is mapped onto it  (such as a "a" on AZERTY mappings, 
 *   or a "q" for QWERTY mappings, etc...).
 * - For example, on AZERTY mappings, key named "Q", "S", "D", and "Z"
 *   are physically located in a way that make them easy to use them
 *   as arrows (left, down, right and up) in a game. 
 *    This is not the case on QWERTY
 *   mappings, so for such a use, you may want to use bx_keycode_pressed()
 *   instead of bx_keyname_pressed().
 */
int bx_keycode_pressed  (bx_key key, unsigned char keycode);

/**
 * Given a key named keyname, returns its keycode,
 * that its physical location on the keyboard with respect to
 * the installed key mapping (AZERTY, QWERTY, ...) 
 *
 * @param keyname a name describing a key
 * @return the keycode of this key, or 0 if no such key exists.
 */
unsigned char bx_keyname_to_keycode (char keyname []);

/**
 * Given a physical location on the keyboard described by keycode,
 * returns its keyname, that is, the attribution of this key with respect
 * the installed key mapping (AZERTY, QWERTY, ...) 
 *
 * @param keycode a keycode describing the physical location of a key. 
 * @return the keyname of this key, or NULL if no such key exists.
 *
 * @remarks
 * - you do not own the returned string, so you must not write over it, 
 *   and you must not call free() on it.
 */
char * bx_keycode_to_keyname (unsigned char keycode);

/**
 * Returns a string listing the names of all keys that are pressed
 * in the given snapshot.
 *
 * @param key a snapshot of the keyboard returned by bx_read_key().
 * @return a string listing the keynames, separated by spaces.
 *
 * @remarks
 * - you do not own the returned string, so you must not write over it, 
 *   and you must not call free() on it.
 * - also, the same string is shared for all the calls to this function,
 *   so you have to strcpy() the results if you want to keep track
 *   of calls older than the last one.
 */
char * bx_keynames (bx_key key);

/**
 * Gets the number of key that are pressed in a given keyboard snapshot.
 *
 * @param key a snapshot of the keyboard returned by bx_read_key().
 * @return the number of keys that are pressed in the snapshot.
 */
int bx_key_number (bx_key key);

/**
 * Sets the autorepeat mode of the keyboard on or off.
 * When on, if a key is kept physically pressed after a certain amount 
 * of time, the system cyclically reports "fake" couples 
 * of release/press states. In bx-lib, autorepeat is off by default,
 * thaty is, bx-lib ignores the "fake" states reported by X11.
 *
 * @param activate sets autorepeat mode on (non zero) or off (zero).
 * @return nothing
 *
 * @remarks
 * - bx-lib never asks X11 to stop to emit "fake" reports.
 *   It only ignores them when it wants to.
 *   If bx-lib did such a thing, it would only be possible at a global level,
 *   that is, other X11 applications would also
 *   be deprived of the "fake" reports", a thing we certainly do not want
 *   (pressing continously the space-bar in a text editor unrelated to 
 *    the program would be without the desired effect for example).
 * - Conversely, bx-lib never force X11 to emit "fake" reports,
 *   it just does not ignore them when it wants to.
 *   So, if another application asks X11 to stop to emit "fake" reports,
 *   or if the user does the same with the unix command "xset r off",
 *   a call to bx_set_autorepeat (1) has no visible effect because we
 *   bx-lib just stops to ignore "fake" reports that never happen.
 */
void bx_set_autorepeat (int activate);

/**
 * Checks whether autorepeat mode for the keyboard is on or off.
 * @return autorepeat on (= 1), or autorepeat off (= 0). 
 */
int  bx_get_autorepeat (void);

/**
 * @}
 */

/*-----------------------------------------------------------------------------
 * mouse control
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup mouse Actively reading the current state of the mouse
 *
 * @{
 */

/**
 * Takes a snapshot of the current state of the mouse (location and buttons).
 * @param win the window to use as referential for the mouse location.
 * @return a instantaneous snapshot of the mouse state 
 */
bx_mouse bx_read_mouse    (bx_window win);

/**
 * Same as bx_mouse_xcanvas().
 */
INLINE int bx_mouse_x       (bx_mouse mouse);

/**
 * Gets the x-location of the mouse, 
 * relative to the canvas of the referential window
 * @param mouse a snapshot of the state of the mouse.
 * @return the x-location.
 */
INLINE int bx_mouse_xcanvas (bx_mouse mouse);

/**
 * Gets the x-location of the mouse, 
 * relative to the top left corner of screen
 * @param mouse a snapshot of the state of the mouse.
 * @return the x-location.
 */
INLINE int bx_mouse_xscreen (bx_mouse mouse);

/**
 * Gets the x-location of the mouse, 
 * relative to the referential window
 * @param mouse a snapshot of the state of the mouse.
 * @return the x-location.
 */
INLINE int bx_mouse_xwindow (bx_mouse mouse);

/**
 * Same as bx_mouse_ycanvas().
 */
INLINE int bx_mouse_y       (bx_mouse mouse);

/**
 * Gets the y-location of the mouse, 
 * relative to the canvas of the referential window
 * @param mouse a snapshot of the state of the mouse.
 * @return the y-location.
 */
INLINE int bx_mouse_ycanvas (bx_mouse mouse);

/**
 * Gets the y-location of the mouse, 
 * relative to the top left corner of screen.
 * @param mouse a snapshot of the state of the mouse.
 * @return the y-location.
 */
INLINE int bx_mouse_yscreen (bx_mouse mouse);

/**
 * Gets the y-location of the mouse, 
 * relative to the referential window.
 * @param mouse a snapshot of the state of the mouse
 * @return the y-location.
 */
INLINE int bx_mouse_ywindow (bx_mouse mouse);

/**
 * Gets the state of mouse button no 1 (usually left), 
 * @param mouse a snapshot of the state of the mouse.
 * @return the state of the button.
 */
INLINE int bx_mouse_b1 (bx_mouse mouse);

/**
 * Gets the state of mouse button no 2 (middle), 
 * @param mouse a snapshot of the state of the mouse.
 * @return the state of the button.
 */
INLINE int bx_mouse_b2 (bx_mouse mouse);

/**
 * Gets the state of mouse button no 3 (usually right), 
 * @param mouse a snapshot of the state of the mouse.
 * @return the state of the button.
 */
INLINE int bx_mouse_b3 (bx_mouse mouse);

/**
 * Gets the state of mouse button no 4 (forward roll of mousewheel), 
 * @param mouse a snapshot of the state of the mouse.
 * @return the state of the button.
 * @remark 
 *  - A mousewheel roll is simulated by X11 by a instantaneous
 *    couple of "press" + "release" events such that the press state can never 
 *    be caught by a snapshot via bx_read_mouse().
 *    Therefore, the use of notification of events 
 *    is necessary to catch a mousewheel roll.
 */
INLINE int bx_mouse_b4 (bx_mouse mouse);

/**
 * Gets the state of mouse button no 5 (backward roll of mousewheel), 
 * @param mouse a snapshot of the state of the mouse.
 * @return the state of the button.
 * @remark 
 * - A mousewheel roll is simulated by X11 by a instantaneous
 *   couple of "press" + "release" events such that the press state can never 
 *   be caught by a snapshot via bx_read_mouse().
 *   Therefore, the use of notification of events 
 *   is necessary to catch a mousewheel roll.
 */
INLINE int bx_mouse_b5 (bx_mouse mouse);

/**
 *  Gets the state of mouse button number b. 
 *  @param mouse a snapshot of the state of the mouse
 *  @param b a number of mouse button in the range from 1 to 5
 *  @return the state of the button.
 */
INLINE int bx_mouse_b (bx_mouse mouse, int b);

/**
 *  Gets the state of the Shift key 
 *  at the time of the snapshot  of the state of the mouse. 
 *  @param mouse a snapshot of the state of the mouse
 *  @return the state of the Shift key.
 */
INLINE int bx_mouse_shift (bx_mouse mouse);

/**
 *  Gets the state of the Ctrl key 
 *  at the time of the snapshot  of the state of the mouse. 
 *  @param mouse a snapshot of the state of the mouse
 *  @return the state of the Ctrl key.
 */
INLINE int bx_mouse_ctrl  (bx_mouse mouse);

/**
 *  Gets the state of the Alt key 
 *  at the time of the snapshot  of the state of the mouse. 
 *  @param mouse a snapshot of the state of the mouse
 *  @return the state of the Alt key.
 */
INLINE int bx_mouse_alt   (bx_mouse mouse);

/**
 *  Gets the state of the AltGr key 
 *  at the time of the snapshot  of the state of the mouse. 
 *  @param mouse a snapshot of the state of the mouse
 *  @return the state of the AltGr key.
 */
INLINE int bx_mouse_altgr (bx_mouse mouse);


/**
 * @}
 */

/*-----------------------------------------------------------------------------
 * events
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup events Being Passively notified of state changes by events 
 * 
 * \brief More advanced programmers in C can be notified of events such 
 * as a change of the state of the mouse 
 * and the keyboard by making some of their own functions be automatically 
 * called by the bx-lib. 
 * This requires the knowledge that functions may be passed
 * as arguments to other functions.
 * The notifications occur when the bx-lib "has the hand"
 * on the system, that is, during the execution of 
 * functions such as bx_wait(), bx_loop() and bx_show_canvas().
 * @{
 */

/**
 * bx_mouse_function is the prototype for functions wanting to be notified
 * on mouse motion, mouse button press, and mouse button release.
 */
typedef void (* bx_mouse_function) (bx_window window, bx_mouse mouse);


/**
 * How to notify a function that a mouse button 
 * is pressed over a given window.
 */
void bx_on_mouse_press    (bx_window window, bx_mouse_function function);

/**
 * How to notify a function that a mouse button 
 * is released over a given window.
 */
void bx_on_mouse_release  (bx_window window, bx_mouse_function function);

/**
 * How to notify a function that the mouse is moving 
 * over a given window.
 */
void bx_on_mouse_move     (bx_window window, bx_mouse_function function);

/**
 * bx_key_function is the prototype for functions wanting to be notified
 * on key press, and key release.
 */
typedef void (* bx_key_function  ) (bx_window window, unsigned char keycode);

/**
 * How to notify a function that a key of the keyboard is pressed 
 * while a given window owns the keyboard focus.
 */
void bx_on_key_press    (bx_window window, bx_key_function function);

/**
 * How to notify a function that a key of the keyboard is released 
 * while a given window owns the keyboard focus.
 */
void bx_on_key_release  (bx_window window, bx_key_function function);


/**
 * @}
 * @}
 */

/*-----------------------------------------------------------------------------
 * directories
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup directory Directories (how to inspect file system easily)
 *
 * @{
 */

/**
 * Opens a directory given a path name.
 * @param path the path name of the directory
 * @return the opened directory
 */

bx_dir  bx_open_dir   (char const path []);

/**
 * Opens a directory given a path format like printf() function families.
 * @param format the printf()-like format yielding the directory path name
 * @param ... the arguments needed by the format specifiers
 * @return the opened directory
 */
bx_dir  bx_open_dir_f (char const format [], ...);

/**
 * Closes an opened directory after usage.
 * This is necessary in order to clean the allocated resources.
 * @param dir the opened directory to close
 * @return nothing.
 */

void    bx_close_dir  (bx_dir dir);

/**
 * Gets the length of an opened  directory, that is, the number of filenames
 * it contains.
 * @param dir the opened directory
 * @return the directory length
 */
int     bx_dir_length (bx_dir dir);


/**
 * Gets the filename at a given location index insde an opened directory.
 * The index should be in the range 0...bx_dir_length()-1.
 * @param dir the opened directory
 * @param index a location in the directory
 * @return the queried filename. This points to a string owned by the function.
 * You cannot free it and its contents change after each call of the function.
 */

char *  bx_dir_filename (bx_dir dir, int index);

/**
 * @}
 */

/*-----------------------------------------------------------------------------
 * SDL Audio...
 *-----------------------------------------------------------------------------
 */

/**
 * @defgroup audio Audio (how to play sounds and music easily)
 *
 * @{
 */

#ifndef BX_AUDIO
#define BX_AUDIO
#endif

#ifdef BX_AUDIO


typedef struct bx_audio_stuff bx_audio_stuff;


/**
 * @brief the type for an audio soundtrack
 */
typedef struct bx_audio { bx_audio_stuff * stuff; } bx_audio;

/**
 * Initializes the audio library.
 * The function is available if the library was compiledwith audio support.
 * No other audio function should be called before this one.
 */

void bx_init_audio (void);

/**
 * Loads an audio soundtrack in WAV format from a filename.
 *
 * @param filename the path to the filename of the WAV file to be loaded 
 * @return the loaded soundtrack 
 */

bx_audio bx_load_audio   (char const filename []);

/**
 * Loads an audio soundtrack in WAV format from a printf()-like 
 * format yielding the filename of the sound file. 
 *
 * @param format the  printf()-like format
 * @apram ... the arguments of the format yielding the filename
 * @return the loaded soundtrack 
 */

bx_audio bx_load_audio_f (char const format [], ...);

/**
 * Plays an audio soundtrack. The track is played in background mixed with
 * other tracks still played, and the function returns immediately. 
 * You can specify the volume, a delay before the sound is played
 * (that is, it will be played in the future) and whether it will be  played
 * only once or repeeatedly ad infinitum.
 *
 * @param audio the soundtrack to play
 * @param volume a volume rate between 0.0 (mute) and 1.0 (max volume)
 * @param millisec the delay in millesconds before the track starts
 * @param ad_infinitum boolean stating whether the sound repeats over and over
 */

void bx_play_audio (bx_audio audio, double volume,  
		    unsigned long millisec, int ad_infinitum); 

#endif

/**
 * @}
 */

/*-----------------------------------------------------------------------------
 * that's all folks
 *-----------------------------------------------------------------------------
 */

#endif
